module geometry

use definitions
use math
use io

contains

!================================================================!
!= newpos sets the new position, and stores the old as r_old	=!
!= r_temp is important when a neutron have had a distchange	=!
!=   because of a void, and is concavechecked afterwards	=!
!================================================================!

subroutine newpos(counter)
implicit none
integer :: counter

r_old(:)=r(:)
r_temp(:)=r(:)
r(:)=r_old(:)+dist*r_direc(:)
if (pathc==1) call path(pathn,counter,r_old(:))

end subroutine newpos

!================================================================!
!= newpos2 is like newpos1, but it doesn't change r_old(:),	=!
!=   this is usefull when correcting path lengths		=!
!= r_temp is important when a neutron have had a distchange	=!
!=   because of a void, and is concavechecked afterwards	=!
!================================================================!

subroutine newpos2(counter)
implicit none
integer :: counter

if (pathc==1) call path(pathn,counter,r(:))
r_temp(:)=r(:)
r(:)=r_old(:)+dist*r_direc(:)

end subroutine newpos2

!================================================================!
!= distchange is a simple tool to change the distance the 	=!
!=    the neutron needs to travel based on intersections with	=!
!=    the void							=!
!================================================================!

subroutine distchange
implicit none

if ( cylinder_v==0) then
  dist=dist+distance(r_inters1,r_inters2)
else if ( cylinder_v==1) then
  dist=dist+distance(r_interc1,r_interc2)
endif

end subroutine distchange

!================================================================!
!= inorout is a simple tool to check if intersections with 	=!
!=  the void is in the simple box or not.			=!
!= Output: ioru=1 : Both intersections in simple box		=!
!= Output: ioru=0 : One or more of the intersection not in box	=!
!================================================================!

subroutine inorout
implicit none

if ( cylinder_v==0) then
 if(r_inters1(3).le.zmax.and.r_inters2(3).le.zmax) then
	ioru=1
 else
	ioru=0
 endif
else 
 if(r_interc1(3).le.zmax.and.r_interc2(3).le.zmax) then
	ioru=1
 else
	ioru=0
 endif
endif

end subroutine inorout


!================================================================!
!= mirror mirrors the direction of the neutron after hitting a	=!
!=    simulation boundary, and calculates the remaining distance=!
!= Has been checked with simple situations			=!
!= There have been some problems with this code since the 	=!
!=  intersections do not have suffiecient accuracy. 		=!
!=  Hence it have been written in a way where this is not	=!
!=  as critical. r is set so r_old will be at the side. 	=!
!= The program assumes r_old is inside the simple box at all	=!
!=  times, if this is not true it will fail and it will tell so	=!
!=  at runtime.							=!
!================================================================!

subroutine mirror
implicit none

if (interside==1) then
 if (dist-distance(r_old,r_interb).ge.-0.00001) then
  dist=dist-distance(r_old,r_interb)
  r_direc(3)=-r_direc(3)
	r(1)=r_interb(1)
	r(2)=r_interb(2)
	r(3)=0.0001
  if(dist.lt.0) mserror=mserror+1
 else
  print*,'mirror error: dist negative'
  stuck=1
  merror=merror+1
 endif
endif

if (interside==2) then
 if (dist-distance(r_old,r_interb).ge.-0.00001) then
  dist=dist-distance(r_old,r_interb)
  r_direc(2)=-r_direc(2)
	r(1)=r_interb(1)
	r(2)=0.0001
	r(3)=r_interb(3)
  if(dist.lt.0) mserror=mserror+1
 else
  print*,'mirror error: dist negative'
  merror=merror+1
  stuck=1
 endif
endif

if (interside==3) then
 if (dist-distance(r_old,r_interb).ge.-0.00001) then
  dist=dist-distance(r_old,r_interb)
  r_direc(2)=-r_direc(2)
	r(1)=r_interb(1)
	r(2)=ymax-0.0001
	r(3)=r_interb(3)
  if(dist.lt.0) mserror=mserror+1
 else
  print*,'mirror error: dist negative'
  merror=merror+1
  stuck=1
 endif
endif

if (interside==4) then
 if (dist-distance(r_old,r_interb).ge.-0.00001) then
  dist=dist-distance(r_old,r_interb)
  r_direc(1)=-r_direc(1)
	r(1)=xmax-0.0001
	r(2)=r_interb(2)
	r(3)=r_interb(3)
  if(dist.lt.0) mserror=mserror+1
 else
  print*,'mirror error: dist negative'
  merror=merror+1
  stuck=1
 endif
endif

if (interside==5) then
 if (dist-distance(r_old,r_interb).ge.-0.00001) then
  dist=dist-distance(r_old,r_interb)
  r_direc(1)=-r_direc(1)
	r(1)=0.0001
	r(2)=r_interb(2)
	r(3)=r_interb(3)
  if(dist.lt.0) mserror=mserror+1
 else
  print*,'mirror error: dist negative'
  merror=merror+1
  stuck=1
 endif
endif

end subroutine mirror

end module geometry