module io

use definitions
use math
use phys
use exprandom

contains

!================================================================!
!= In the subroutine message a starting message is produced.	=!
!=  This is needed when running longer simulations so settings	=!
!=  can be checked.						=!
!================================================================!

subroutine message
implicit none
print *,'==================STARTUP MESSAGE===================='
print *,'This program simulates a neutron moderator.'
print *,'Please see main.f90 and moderator.f90 for options.'
print *,'The program is not in test mode and will run'
print *,'  with randomly generated initial neutrons.'
print *,'Options is as follows:'
print *,'nmax=',nmax
if (isotropic==1) then
print *,'Scattering is simulated as isotropic.'
print *,'Energy decreases nonrandomly.'
else if (isotropic==0) then
print *,'Elastic scattering is simulated.'
print *,'Energy decreases randomly'
endif

if (absorption==1) then
print *,'Absorption is simulated'
print *,'All neutrons start with thermal energy'
endif

if (flat==1) then
 print *,'A FLAT moderator surface is used.'
endif

if (cylinder_v==1.and.flat==0) then
 print *,'A CYLINDER void in the moderator surface is used.'
endif

if (cylinder_v==0.and.flat==0) then
print *,'A SPHERE void in the moderator surface is used.'
endif

if (cylinder==1) then
 print *,'WARNING, SIMULATING BIGGER MAIN MODERATOR!'
endif
print *,'==================RUNNING PROGRAM===================='

end subroutine message

!================================================================!
!= Endmessage gives runtime output after the simulations is done=!
!= Small errors is counted throughout the simulation and the    =!
!=  total numbers is shown here. If these seem to big to change	=!
!=  any statistics, the data should not be used.		=!
!= With the current state of the simulation, runs at 30 million =!
!=  simulated neutrons does not produce errors.			=!
!================================================================!

subroutine endmessage
implicit none

print*,'data written'
if (absorption==1) then
print*,'number off absorped n :',absorped
endif
print*,'number of small errors:',terror+berror
print*,'box intersect errors  :',berror
print*,'mirror dist<0 errors  :',merror
print*,'mirror dist=0 events  :',mserror
print*,'saferun       errors  :',terror

end subroutine endmessage

!================================================================!
!= Output is used everytime a neutrons data should be written to=!
!=  the temporary data file stored in ram.			=!
!= output is tested with writetodisk   				=!
!================================================================!

subroutine output(counter,counter2)
implicit none
integer		:: counter,counter2

if (pathc==1) call path(pathn,counter2,r_old(:))
if (pathc==1) call path(pathn,counter2+1,r(:))

results(n,:)=(/r_old(1),r_old(2),r_old(3),&
		r_direc(1),r_direc(2),r_direc(3), &
		E,tdist,real(interact), real(boundary), &
		real(error),real(stuck),real(fail),real(k)/)

end subroutine output

!================================================================!
!= outputm writes another set of output				=!
!================================================================!

subroutine outputm(counter,counter2)
implicit none
integer		:: counter,counter2

if (pathc==1) call path(pathn,counter2,r_old(:))
if (pathc==1) call path(pathn,counter2+1,r(:))

resultsm(m,:)=(/r_old(1),r_old(2),r_old(3),&
		r_direc(1),r_direc(2),r_direc(3), &
		E,tdist,real(interact), real(boundary), &
		real(error),real(stuck),real(fail),real(k)/)

end subroutine outputm

!================================================================!
!= writetodisk writes data from the ram and to the disk after   =!
!=  all wanted neutrons is simulated. This increases performance=!
!= since harddisk search times are significant, but not 	=!
!=  not relevant when writing one long strip of data instead of =!
!=  many smaller.						=!
!= The data file is called data.dat and has header lines from   =!
!=  the info variabel. In this way datafiles will not get mixed =!
!=  up.								=!
!= writetodisk is tested with output   				=!
!================================================================!

subroutine writetodisk
implicit none
integer :: k1,notwritten
real,dimension(16) :: info

notwritten=0

do k1=1,nmax
 if (results(k1,13).gt.0.5.or.results(k1,12).gt.0.5.or.results(k1,11).gt.0.5) then
 notwritten=notwritten+1
 endif
enddo

info=(/xmax,ymax,zmax,r0_sphere,r_sphere,E1,Etermisk,real(nmax-notwritten)&
      ,real(k),real(test),real(isotropic),real(cylinder) &
      ,real(cylinder_v),real(flat)/)

notwritten=0

open(10,file='data.dat')

 write(10,'(8x(1x,g14.6))')  info(:)

close(10)

open(10,file='data.dat',position='append')

do k1=1,nmax
 if (results(k1,13).lt.0.5.and.results(k1,12).lt.0.5.and.results(k1,11).lt.0.5) then
  write(10,'(11(1x,g18.10))') results(k1,14),results(k1,1:10)
 else
  notwritten=notwritten+1
 endif
end do

close(10)

 if (notwritten==0) print*, 'Every event in datafile'
 if (notwritten.ne.0) print*,'Number of events discarded:',notwritten

end subroutine writetodisk

!================================================================!
!= This version of writetodisk takes care of datta from outputm =!
!= writetodisk is tested with output   				=!
!================================================================!

subroutine writetodiskm
implicit none
integer :: k1,notwritten
real,dimension(16) :: info

notwritten=0

do k1=1,mmax
 if (resultsm(k1,13).gt.0.5.or.resultsm(k1,12).gt.0.5.or.resultsm(k1,11).gt.0.5) then
 notwritten=notwritten+1
 endif
enddo

info=(/xmax,ymax,zmax,r0_sphere,r_sphere,E1,Etermisk,real(mmax-notwritten)&
      ,real(k),real(test),real(isotropic),real(cylinder) &
      ,real(cylinder_v),real(flat)/)

notwritten=0

open(10,file='datam.dat')

 write(10,'(8x(1x,g14.6))')  info(:)

close(10)

open(10,file='datam.dat',position='append')

do k1=1,mmax
 if (resultsm(k1,13).lt.0.5.and.resultsm(k1,12).lt.0.5.and.resultsm(k1,11).lt.0.5) then
  write(10,'(11(1x,g18.10))') resultsm(k1,14),resultsm(k1,1:10)
 else
  notwritten=notwritten+1
 endif
end do

close(10)

 if (notwritten==0) print*, 'Every mirror event in datafile'
 if (notwritten.ne.0) print*,'Number of mirror events discarded/nothappend:',notwritten

end subroutine writetodiskm


!================================================================!
!= Pathinit initializes a path data file. On a typical run a few=!
!=  simulated neutrons will have their path recorded.		=!
!= This routine starts a path file.				=!
!= pathinit overwrites nr file, so path can			=!
!=   use append without risking to continue			=!
!=   an old file						=!
!================================================================!

subroutine pathinit(nr,neutron)
implicit none
integer		 :: nr,neutron
real,dimension(3):: vec

if (nr.lt.10)then
name='path_'//char(48)//char(nr+48)//'.dat'
else if (nr.ge.10.and.nr.lt.20) then
name='path_'//char(49)//char(nr-10+48)//'.dat'
endif

open(20,file=name)
write(20,'(11(1x,g18.10))') nr,neutron,xmax,ymax,zmax,r0_sphere,r_sphere,E1,Etermisk
close(20)


end subroutine pathinit

!================================================================!
!= Path is a low performance writer which writes the current	=!
!=  position of the neutron for each iteration. Since few	=!
!=  neutrons are followed in this way it is not important for	=!
!=  overall performance						=!
!================================================================!

subroutine path(nr,counter,vec)
implicit none
integer		 :: counter,nr
real,dimension(3):: vec

if (nr.lt.10)then
name='path_'//char(48)//char(nr+48)//'.dat'
else if (nr.ge.10.and.nr.lt.20) then
name='path_'//char(49)//char(nr-10+48)//'.dat'
endif

open(20,file=name,position='append')
write(20,'(6(1x,g18.10))') counter,vec(:),dist,E
close(20)

end subroutine path

!================================================================!
!= Teststartio takes care of needed io for test mode. 		=!
!= Test mode saves the path of every neutron.			=!
!=  It was used mainly for debugging where it can be used to	=!
!=   create specific special cases and checking if the program	=!
!=   handles the situation correctly.				=!
!= The routine are not quite up to date though, so options are  =!
!=  not well supported in the io part, but can be used with	=!
!=  caution and a little rewriting in main.f90			=!
!================================================================!

subroutine teststartio
implicit none

print*,'========== TEST MODE ========================================'
print*,'In test mode the user can specify a start point and direction'
print*,'  and watch the path of the neutron.'
print*,'Test mode has not been updated to show accurate information  '
print*,' in cases with cylinder_v=1 or cylinder=1		     '
print*,'============================================================='
print*,'Do you want to specify the first dist ? (1 for yes, 0 for no)'
read*,testdist
print*,'============================================================='
print*,'The moderator is a box from (0,0,0) to'
print*,'xmax',xmax
print*,'ymax',ymax
print*,'zmax',zmax
print*,'============================================================='
if (flat==0) then
print*,'The moderator has a void in it, shaped like a sphere with'
print*,'center x', r0_sphere(1)
print*,'center y', r0_sphere(2)
print*,'center z', r0_sphere(3)
print*,'radius  ', r_sphere
else
print*,'The moderator surface is flat'
endif
print*,'============================================================='
print*,'Insert start r_old coordinates'
read*,r_old(:)
print*,'============================================================='
print*,'Insert start r coordinates'
read*,r(:)
print*,'============================================================='
print*,'Insert start direction as xyz vector. Will be normalized'
read*,r_direc(:)

r_direc=r_direc/length(r_direc)

print*,'normalized r_direc', r_direc
print*,'============================================================='
print*,'Insert the neutron energy in electron volts (thermal=0.025eV'
read*,E
print*,'============================================================='

 call cross_section
 sigma=7.96E19*cross

if (testdist==1) then
 print*,'Insert first distance'
 read*,dist
 print*,'============================================================='
else
 dist=expdev(seed3,sigma)
endif
print*,'Insert 3 negative integers for random seeds'
read*,seed1,seed2,seed3
print*,'============================================================='

end subroutine teststartio

!================================================================!
!= testendio asks if another test run is wanted			=!
!================================================================!

subroutine testendio

print*,'============================================================='
print*,'Do you want to make another test path ? (1 for yes )'
read*,testdone
print*,'============================================================='

end subroutine testendio

end module io