Program main

!========================================================!
!= Main part of program for simulating neutron moderator=!
!=  material. There is a number of options in this file =!
!=  with information under each explanation. 		=!
!= It should be noted the program works with units:	=!
!=  length in mm, energy in eV				=!
!= Additional options can be found in the files:	=!
!=  definition.f90					=!
!=    nmax: number of neutrons outputted from the top	=!
!=    mfactor: number of neutrons outputted from sides  =!
!=		( a maximum, set in units of nmax )	=!
!=    Ehydro: energy of hydrogen atoms in the moderator =!
!=  moderator.f90					=!
!=    Geometry of the different shapes supported	=!
!=	 can be changed in this file			=!
!=  mainloop.f90 needs to be edited for neutron mirrors =!
!=    to be used. This requires knowledge of interside, =!
!=    which can be found in interbox in math.f90 	=!
!=							=!
!= The program has a test mode which can be helpfull to	=!
!=  understand the programs method. To enabel it, change=!
!=  the variabel test to 1, then compile and run. 	=!
!=  At runtime a guide in starting a neutron will be 	=!
!=  given, and the path data will be saved. 		=!
!= If the format seems unintuitive, io.f90 can be viewd =!
!=  for help.						=!
!========================================================!

use random
use io
use definitions
use exprandom
use ranphys
use moderator
use math
use geometry
use mainloop

implicit none

seed1=-2935
seed2=-9475
seed3=-5475
seed4=-4935
seed5=-2135
seed6=-7696
seed7=-3592
seed8=-5923
seed9=-1030
seed10=-14849

imax=50000
l=0
l2=0
pathn=0 
terror=0
berror=0
merror=0
absorped=0
E1=5.7E6
Etermisk=Ehydro
n=0
k=0
m=0


E=Etermisk
call cross_section
sigma2=7.515E19*cross
E=E1
call cross_section
sigma1=7.515E19*cross
sigmaabs=0.0025

!========================================================!
!= Set test to 0 to simulate nmax neutrons in moderator	=!
!= set test to 1 to test system, io at runtime	 	=!
!========================================================!
			test=0
!========================================================!
!= Set isotropic=1 to use isotropic scattering		=!
!= set isotropic=0 to use non isotropic scattering	=!
!========================================================!
			isotropic=0
!========================================================!
!= Set cylinder=1 if you want to test for a big box  	=!
!=  resembling our main moderator.			=!
!= Neutron start will be a point source at the center	=!
!=  with random direction and energy 5.7MeV		=!
!========================================================!
			cylinder=0
!========================================================!
!= Set cylinder_v=1 if you want to run the program with	=!
!=  a cylindrical void					=!
!= Set cylinder_v=0 if you want to run the program with =!
!=  a spherical void.					=!
!========================================================!
			cylinder_v=0
!========================================================!
!= If flat =1 the sphere and cylinder will not intersect=!
!=  the moderator, making the moderator flat.		=!
!========================================================!
			flat=0
!========================================================!
!= To test the effects of absorption, absorption can be =!
!=  enabled, but this will limit the simulation to one	=!
!=  energy level, chosen to be thermal. 		=!
!= absorption=1 to enable				=!
!========================================================!
			absorption=0

if (test==0) then 
!========================================================!	
!= 		MAIN PROGRAM START			=!
!========================================================!

 call message
 call setmoderator

!========================================================!	
!= 		MAIN LOOP START				=!
!========================================================!

do

!========================================================!	
!= 		PROGRESS SYSTEM				=!
!========================================================!

	if (l==int(nmax/20)) then
		print*,100*n/nmax,'percent done'
		l=0
	endif

!========================================================!
!= 		NEUTRON PATH SELECTOR			=!
!========================================================!

	if (l2==int(nmax/10)) then
		pathc=1
		pathn=pathn+1
		call pathinit(pathn,k)
		l2=0
	else
		pathc=0
	endif
!========================================================!
!= 		END SYSTEM 				=!
!========================================================!

if (n==nmax) then
 print*,'n=nmax! happend at k=',k
 exit
endif

 k=k+1
!========================================================!
!= 		SIMULATE NEUTRON			=!
!========================================================!
 interact=1
 tdist=0
 boundary=0
 error=0
 stuck=0
 fail=0

 call newneutron(seed4,seed5,seed3)
 call simulate

enddo

!========================================================!	
!= 		MAIN LOOP END				=!
!========================================================!

call writetodisk
call writetodiskm
call endmessage

!========================================================!	
!= 		MAIN PROGRAM END			=!
!========================================================!

else if (test==1) then

call setmoderator

do 
	pathc=1
	pathn=pathn+1
	call pathinit(pathn,pathn)

call teststartio

 call cross_section

 sigma=7.96E19*cross

	 call path(pathn,0,r_old(:))

call simulatetest
call endmessage
call testendio
if (testdone.ne.1) exit
end do

end if

end program main