module math

use definitions

contains

!==============================================!
!= Calculates the length of a vector	      =!
!==============================================!

function length(r1)
implicit none
real,dimension(3) 	:: r1
real			:: length

length=sqrt( r1(1)**2 + r1(2)**2 + r1(3)**2)
return

end function length

!==============================================!
!= Calculates the cross product of two vectors=!
!==============================================!

function crossp(r1,r2)
implicit none
real,dimension(3) 	:: r1,r2,crossp

crossp(1)=r1(2)*r2(3)-r2(2)*r1(3)
crossp(2)=r1(3)*r2(1)-r2(3)*r1(1)
crossp(3)=r1(1)*r2(2)-r2(1)*r1(2)

return

end function crossp

!==============================================!
!= Calculates the distance between two points =!
!==============================================!

function distance(r1,r2)
implicit none
real,dimension(3) 	:: r1,r2
real			:: distance

distance=sqrt( ( r1(1)-r2(1) )**2 + ( r1(2)-r2(2) )**2 + ( r1(3)-r2(3) )**2)
return

end function distance

!================================================================!
!= interbox checks the intersection between the line made by	=!
!=  the old neutron position and direction and the simple box.	=!
!= The simple box is the box as it would be without void.	=!
!= Output is the intersection r_interb and interside integer	=!
!=								=!
!= interside = 1 if intersection is on ingoing side  (z=0)	=!
!= interside = 2 if intersection is on buttom side   (y=0)	=!
!= interside = 3 if intersection is on top side      (y=ymax)	=!
!= interside = 4 if intersection is on left side     (x=xmax)	=!
!= interside = 5 if intersection is on right side    (x=0)	=!
!= interside = 6 if intersection is on outgoing side (z=zmax)	=!
!=								=!
!= When neutrons hit a corner og edge where both intersections  =!
!= could be considered valid, the side lowest number is chosen. =!
!= This is done to be 100% sure neutrons with interside=6 are	=!
!=  outgoing							=!
!================================================================!

subroutine interbox
implicit none
real			:: t
real,dimension(3) 	:: rtest
integer			:: done


done=0
interside=0

! Assuming intersection is on ingoing side
if (done==0) then
 if (r_direc(3).ne.0.) then
  t=(-r_old(3))/r_direc(3)
  rtest(:)=r_old(:)+t*r_direc(:)
  if (t.ge.0..and.t.le.dist) then
     if (rtest(1).ge.0..and.rtest(1).le.xmax.and.rtest(2).ge.0..and.rtest(2).le.ymax) then
	r_interb=rtest
	interside=1
	done=1
     endif
  endif
 endif
endif

! Assuming intersection is on buttom side
if (done==0) then
 if (r_direc(2).ne.0.) then
  t=(-r_old(2))/r_direc(2)
  rtest(:)=r_old(:)+t*r_direc(:)
  if (t.ge.0..and.t.le.dist) then
    if (rtest(1).ge.0..and.rtest(1).le.xmax.and.rtest(3).ge.0..and.rtest(3).le.zmax) then
	r_interb=rtest
	interside=2
	done=1
    endif
  endif
 endif
endif

! Assuming intersection is on top side
if (done==0) then
 if (r_direc(2).ne.0.) then
  t=(ymax-r_old(2))/r_direc(2)
  rtest(:)=r_old(:)+t*r_direc(:)
  if (t.ge.0..and.t.le.dist) then
    if (rtest(1).ge.0..and.rtest(1).le.xmax.and.rtest(3).ge.0..and.rtest(3).le.zmax) then 
	r_interb=rtest
	interside=3
	done=1
    endif
  endif
 endif
endif

! Assuming intersection is on left side
if (done==0) then
 if (r_direc(1).ne.0.) then
  t=(xmax-r_old(1))/r_direc(1)
  rtest(:)=r_old(:)+t*r_direc(:)
  if (t.ge.0..and.t.le.dist) then
    if (rtest(2).ge.0..and.rtest(2).le.ymax.and.rtest(3).ge.0..and.rtest(3).le.zmax) then 
	r_interb=rtest
	interside=4
	done=1
    endif
  endif
 endif
endif

! Assuming intersection is on right side
if (done==0) then
 if (r_direc(1).ne.0.) then
  t=(-r_old(1))/r_direc(1)
  rtest(:)=r_old(:)+t*r_direc(:)
  if (t.ge.0..and.t.le.dist) then
    if (rtest(2).ge.0..and.rtest(2).le.ymax.and.rtest(3).ge.0..and.rtest(3).le.zmax) then
	r_interb=rtest
	interside=5
	done=1
    endif
  endif
 endif
endif

! Assuming intersection is on outgoing side
if (done==0) then
 if (r_direc(3).ne.0.) then
  t=(zmax-r_old(3))/r_direc(3)
  rtest(:)=r_old(:)+t*r_direc(:)
  if (t.ge.0..and.t.le.dist) then
    if (rtest(1).ge.0..and.rtest(1).le.xmax.and.rtest(2).ge.0..and.rtest(2).le.ymax) then
	r_interb=rtest
	interside=6
	done=1
    endif
  endif
 endif
endif

if (interside==0) then
  print*,'Error in box intersect'
  berror=berror+1
  stuck=1
endif

end subroutine interbox

!================================================================!
!= intersphere checks the intersection between the line made by	=!
!=   r_old and r_direc and the sphere disribing the void in the =!
!=   moderator.							=!
!= Outputs are r_inters1 and r_inters2				=!
!= u1 and u2 is stored as global varables, so other routines    =!
!=   can check if the solutions are in past or future for the   =!
!=   neutron							=!
!================================================================!

subroutine intersphere
implicit none
real,dimension(3)	:: P1,P2
real			:: a,b,c

P1(:)=r_old(:)
P2(:)=r_old(:)+r_direc(:)

! The intersection is the solution to the second order equation with coefficents:

a=(P2(1)-P1(1))**2+(P2(2)-P1(2))**2+(P2(3)-P1(3))**2

b=2*( ( P2(1)-P1(1) )*( P1(1)-r0_sphere(1) ) +&
      ( P2(2)-P1(2) )*( P1(2)-r0_sphere(2) ) +&
      ( P2(3)-P1(3) )*( P1(3)-r0_sphere(3) ) )

c=r0_sphere(1)**2+r0_sphere(2)**2+r0_sphere(3)**2 + &
	P1(1)**2 + P1(2)**2 + P1(3)**2 - &
	2.*(r0_sphere(1)*P1(1) + r0_sphere(2)*P1(2) + r0_sphere(3)*P1(3)) &
	- r_sphere**2

 u1=(-b-sqrt(b**2.-4.*a*c))/(2.*a)
 u2=(-b+sqrt(b**2.-4.*a*c))/(2.*a)

if (b**2.-4.*a*c.lt.0) then
  print*, 'intersphere error, no solutions'
else
  r_inters1(:)=P1(:)+u1*(P2(:)-P1(:))
  r_inters2(:)=P1(:)+u2*(P2(:)-P1(:))
endif


end subroutine intersphere

!================================================================!
!= intercylinder checks the intersection between the line made 	=!
!=   by r_old and r_direc and the cylinder disribing the start  =!
!=   of the moderator system					=!
!= Outputs are r_interc1 and r_interc2				=!
!= u3 and u4 is stored as global varables, so other routines    =!
!=   can check if the solutions are in past or future for the   =!
!=   neutron							=!
!================================================================!

subroutine intercylinder
implicit none
real :: a,b,c,t
integer :: c1done,c2done
real,dimension(3) :: rtest

c1done=0
c2done=0

 a=r_direc(1)**2+r_direc(2)**2
 b=2.*( r_direc(1)*(r_old(1)-ccentrum(1)) + r_direc(2)*(r_old(2)-ccentrum(2)) )
 c= r_old(1)**2 + r_old(2)**2 - 2.*( r_old(1)*ccentrum(1) + r_old(2)*ccentrum(2) ) &
				- r_cylinder**2+ccentrum(1)**2+ccentrum(2)**2


 u3=(-b-sqrt(b**2.-4.*a*c))/(2.*a)
 u4=(-b+sqrt(b**2.-4.*a*c))/(2.*a)

if (b**2.-4.*a*c.lt.0) then
   print*, 'intercylinder error, no solutions ( box error )', posstatus
   berror=berror+1
else
 if (r_old(3)+u3*r_direc(3).le.czmax.and. & 
	r_old(3)+u3*r_direc(3).ge.czmin) then
   r_interc1(:)=r_old+u3*r_direc
   c1done=1
 endif
 if (r_old(3)+u4*r_direc(3).le.czmax.and. & 
 	r_old(3)+u4*r_direc(3).ge.czmin) then
   r_interc2(:)=r_old+u4*r_direc
   c2done=1
 endif

 if (c1done==0) then
   t=(czmax-r_old(3))/r_direc(3)
   rtest(:)=r_old(:)+t*r_direc(:)
     if ( (rtest(1)-ccentrum(1))**2 + (rtest(2)-ccentrum(2))**2.le.r_cylinder**2) then 
	 r_interc1=rtest
	 u3=t
	 c1done=1
     endif

   t=(czmin-r_old(3))/r_direc(3)
   rtest(:)=r_old(:)+t*r_direc(:)
     if ( (rtest(1)-ccentrum(1))**2 + (rtest(2)-ccentrum(2))**2.le.r_cylinder**2) then 
	 r_interc1=rtest
	 u3=t
	 c1done=1
     endif
 endif

 if (c2done==0) then
   t=(czmin-r_old(3))/r_direc(3)
   rtest(:)=r_old(:)+t*r_direc(:)
     if ( (rtest(1)-ccentrum(1))**2 + (rtest(2)-ccentrum(2))**2.le.r_cylinder**2) then 
	 r_interc2=rtest
	 u4=t
	 c2done=1
     endif

   t=(czmax-r_old(3))/r_direc(3)
   rtest(:)=r_old(:)+t*r_direc(:)
     if ( (rtest(1)-ccentrum(1))**2 + (rtest(2)-ccentrum(2))**2.le.r_cylinder**2) then 
 	 r_interc2=rtest
	 u4=t
	 c2done=1
     endif
 endif

 if (c1done==0.or.c2done==0) then
  print*,'cyllinder intersect error, no solution'
  berror=berror+1
 endif

endif

end subroutine intercylinder

end module math