module random

contains

!================================================================!
!= Random functions copyed from numerical recipes. 		=!
!= ran1 is primarily used, but it does require ran0.		=!
!= The random functions have been checked and seems to be what	=!
!=  is expected.						=!
!================================================================!

function ran0(idum)
implicit none
Integer :: idum
real    :: ran0
integer,parameter :: IA=16807,IM=2147483647,IQ=127773,IR=2836,MASK=123459876
real,parameter    :: AM=1./IM
Integer k
idum=ieor(idum,MASK)
k=idum/IQ
idum=IA*(idum-k*IQ)-IR*k
if (idum.lt.0) idum=idum+IM
ran0=AM*idum
idum=ieor(idum,MASK)
return

end function ran0

function ran1dist(idum)
implicit none
integer,parameter :: IA=16807,IM=2147483647,IQ=127773,IR=2836,NTAB=32
integer,parameter :: NDIV=1+(IM-1)/NTAB
real,parameter    :: AM=1./IM,EPS=1.2e-7,RNMX=1.-eps
real		  :: ran1dist
integer 	  :: j,k,iv(NTAB),iy,idum
SAVE iv,iy
DATA iv /NTAB*0/, iy /0/
if (idum.le.0.or.iy.eq.0) then 
   idum=max(-idum,1)
   do j=NTAB+8,1,-1
	k=idum/IQ
	idum=IA*(idum-k*IQ)-IR*k
	if (idum.lt.0) idum=idum+IM
	if (j.le.NTAB) iv(j)=idum
   enddo
   iy=iv(1)
endif
k=idum/IQ
idum=IA*(idum-k*IQ)-IR*k
if (idum.lt.0) idum=idum+IM
j=1+iy/NDIV
iy=iv(j)
iv(j)=idum
ran1dist=min(AM*iy,RNMX)
return

end function ran1dist

function ran1abs(idum)
implicit none
integer,parameter :: IA=16807,IM=2147483647,IQ=127773,IR=2836,NTAB=32
integer,parameter :: NDIV=1+(IM-1)/NTAB
real,parameter    :: AM=1./IM,EPS=1.2e-7,RNMX=1.-eps
real		  :: ran1abs
integer 	  :: j,k,iv(NTAB),iy,idum
SAVE iv,iy
DATA iv /NTAB*0/, iy /0/
if (idum.le.0.or.iy.eq.0) then 
   idum=max(-idum,1)
   do j=NTAB+8,1,-1
	k=idum/IQ
	idum=IA*(idum-k*IQ)-IR*k
	if (idum.lt.0) idum=idum+IM
	if (j.le.NTAB) iv(j)=idum
   enddo
   iy=iv(1)
endif
k=idum/IQ
idum=IA*(idum-k*IQ)-IR*k
if (idum.lt.0) idum=idum+IM
j=1+iy/NDIV
iy=iv(j)
iv(j)=idum
ran1abs=min(AM*iy,RNMX)
return

end function ran1abs

function ran1theta(idum)
implicit none
integer,parameter :: IA=16807,IM=2147483647,IQ=127773,IR=2836,NTAB=32
integer,parameter :: NDIV=1+(IM-1)/NTAB
real,parameter    :: AM=1./IM,EPS=1.2e-7,RNMX=1.-eps
real		  :: ran1theta
integer 	  :: j,k,iv(NTAB),iy,idum
SAVE iv,iy
DATA iv /NTAB*0/, iy /0/
if (idum.le.0.or.iy.eq.0) then 
   idum=max(-idum,1)
   do j=NTAB+8,1,-1
	k=idum/IQ
	idum=IA*(idum-k*IQ)-IR*k
	if (idum.lt.0) idum=idum+IM
	if (j.le.NTAB) iv(j)=idum
   enddo
   iy=iv(1)
endif
k=idum/IQ
idum=IA*(idum-k*IQ)-IR*k
if (idum.lt.0) idum=idum+IM
j=1+iy/NDIV
iy=iv(j)
iv(j)=idum
ran1theta=min(AM*iy,RNMX)
return

end function ran1theta

function ran1phi(idum)
implicit none
integer,parameter :: IA=16807,IM=2147483647,IQ=127773,IR=2836,NTAB=32
integer,parameter :: NDIV=1+(IM-1)/NTAB
real,parameter    :: AM=1./IM,EPS=1.2e-7,RNMX=1.-eps
real		  :: ran1phi
integer 	  :: j,k,iv(NTAB),iy,idum
SAVE iv,iy
DATA iv /NTAB*0/, iy /0/
if (idum.le.0.or.iy.eq.0) then 
   idum=max(-idum,1)
   do j=NTAB+8,1,-1
	k=idum/IQ
	idum=IA*(idum-k*IQ)-IR*k
	if (idum.lt.0) idum=idum+IM
	if (j.le.NTAB) iv(j)=idum
   enddo
   iy=iv(1)
endif
k=idum/IQ
idum=IA*(idum-k*IQ)-IR*k
if (idum.lt.0) idum=idum+IM
j=1+iy/NDIV
iy=iv(j)
iv(j)=idum
ran1phi=min(AM*iy,RNMX)
return

end function ran1phi

function ran1x(idum)
implicit none
integer,parameter :: IA=16807,IM=2147483647,IQ=127773,IR=2836,NTAB=32
integer,parameter :: NDIV=1+(IM-1)/NTAB
real,parameter    :: AM=1./IM,EPS=1.2e-7,RNMX=1.-eps
real		  :: ran1x
integer 	  :: j,k,iv(NTAB),iy,idum
SAVE iv,iy
DATA iv /NTAB*0/, iy /0/
if (idum.le.0.or.iy.eq.0) then 
   idum=max(-idum,1)
   do j=NTAB+8,1,-1
	k=idum/IQ
	idum=IA*(idum-k*IQ)-IR*k
	if (idum.lt.0) idum=idum+IM
	if (j.le.NTAB) iv(j)=idum
   enddo
   iy=iv(1)
endif
k=idum/IQ
idum=IA*(idum-k*IQ)-IR*k
if (idum.lt.0) idum=idum+IM
j=1+iy/NDIV
iy=iv(j)
iv(j)=idum
ran1x=min(AM*iy,RNMX)
return

end function ran1x

function ran1y(idum)
implicit none
integer,parameter :: IA=16807,IM=2147483647,IQ=127773,IR=2836,NTAB=32
integer,parameter :: NDIV=1+(IM-1)/NTAB
real,parameter    :: AM=1./IM,EPS=1.2e-7,RNMX=1.-eps
real		  :: ran1y
integer 	  :: j,k,iv(NTAB),iy,idum
SAVE iv,iy
DATA iv /NTAB*0/, iy /0/
if (idum.le.0.or.iy.eq.0) then 
   idum=max(-idum,1)
   do j=NTAB+8,1,-1
	k=idum/IQ
	idum=IA*(idum-k*IQ)-IR*k
	if (idum.lt.0) idum=idum+IM
	if (j.le.NTAB) iv(j)=idum
   enddo
   iy=iv(1)
endif
k=idum/IQ
idum=IA*(idum-k*IQ)-IR*k
if (idum.lt.0) idum=idum+IM
j=1+iy/NDIV
iy=iv(j)
iv(j)=idum
ran1y=min(AM*iy,RNMX)
return

end function ran1y

function ran1E1(idum)
implicit none
integer,parameter :: IA=16807,IM=2147483647,IQ=127773,IR=2836,NTAB=32
integer,parameter :: NDIV=1+(IM-1)/NTAB
real,parameter    :: AM=1./IM,EPS=1.2e-7,RNMX=1.-eps
real		  :: ran1E1
integer 	  :: j,k,iv(NTAB),iy,idum
SAVE iv,iy
DATA iv /NTAB*0/, iy /0/
if (idum.le.0.or.iy.eq.0) then 
   idum=max(-idum,1)
   do j=NTAB+8,1,-1
	k=idum/IQ
	idum=IA*(idum-k*IQ)-IR*k
	if (idum.lt.0) idum=idum+IM
	if (j.le.NTAB) iv(j)=idum
   enddo
   iy=iv(1)
endif
k=idum/IQ
idum=IA*(idum-k*IQ)-IR*k
if (idum.lt.0) idum=idum+IM
j=1+iy/NDIV
iy=iv(j)
iv(j)=idum
ran1E1=min(AM*iy,RNMX)
return

end function ran1E1

function ran1E2(idum)
implicit none
integer,parameter :: IA=16807,IM=2147483647,IQ=127773,IR=2836,NTAB=32
integer,parameter :: NDIV=1+(IM-1)/NTAB
real,parameter    :: AM=1./IM,EPS=1.2e-7,RNMX=1.-eps
real		  :: ran1E2
integer 	  :: j,k,iv(NTAB),iy,idum
SAVE iv,iy
DATA iv /NTAB*0/, iy /0/
if (idum.le.0.or.iy.eq.0) then 
   idum=max(-idum,1)
   do j=NTAB+8,1,-1
	k=idum/IQ
	idum=IA*(idum-k*IQ)-IR*k
	if (idum.lt.0) idum=idum+IM
	if (j.le.NTAB) iv(j)=idum
   enddo
   iy=iv(1)
endif
k=idum/IQ
idum=IA*(idum-k*IQ)-IR*k
if (idum.lt.0) idum=idum+IM
j=1+iy/NDIV
iy=iv(j)
iv(j)=idum
ran1E2=min(AM*iy,RNMX)
return

end function ran1E2

!================================================================!
!= rancirc picks a random point on a circle with radius and 	=!
!=  angle. Needs the rancirc1 and 2 which uses ran1		=!
!================================================================!

function rancirc(idum1,idum2)

implicit none
integer             :: idum1, idum2
real                :: dum1, dum2
real, dimension(2)  :: rancirc

do
 dum1 = ran1circ_1(idum1)
 dum2 = ran1circ_2(idum2)
 if ( dum1 == 0. .or. dum2 == 0. ) then
 else
  exit
 endif
enddo

rancirc(1) = sqrt(dum2)         !r value (on unit circle)
rancirc(2) = 2*3.14159 * dum1   !theta value

return

end function rancirc

function ran1circ_1(idum)
implicit none
integer,parameter :: IA=16807,IM=2147483647,IQ=127773,IR=2836,NTAB=32
integer,parameter :: NDIV=1+(IM-1)/NTAB
real,parameter    :: AM=1./IM,EPS=1.2e-7,RNMX=1.-eps
real		  :: ran1circ_1
integer 	  :: j,k,iv(NTAB),iy,idum
SAVE iv,iy
DATA iv /NTAB*0/, iy /0/
if (idum.le.0.or.iy.eq.0) then 
   idum=max(-idum,1)
   do j=NTAB+8,1,-1
	k=idum/IQ
	idum=IA*(idum-k*IQ)-IR*k
	if (idum.lt.0) idum=idum+IM
	if (j.le.NTAB) iv(j)=idum
   enddo
   iy=iv(1)
endif
k=idum/IQ
idum=IA*(idum-k*IQ)-IR*k
if (idum.lt.0) idum=idum+IM
j=1+iy/NDIV
iy=iv(j)
iv(j)=idum
ran1circ_1=min(AM*iy,RNMX)
return

end function ran1circ_1

function ran1circ_2(idum)
implicit none
integer,parameter :: IA=16807,IM=2147483647,IQ=127773,IR=2836,NTAB=32
integer,parameter :: NDIV=1+(IM-1)/NTAB
real,parameter    :: AM=1./IM,EPS=1.2e-7,RNMX=1.-eps
real		  :: ran1circ_2
integer 	  :: j,k,iv(NTAB),iy,idum
SAVE iv,iy
DATA iv /NTAB*0/, iy /0/
if (idum.le.0.or.iy.eq.0) then 
   idum=max(-idum,1)
   do j=NTAB+8,1,-1
	k=idum/IQ
	idum=IA*(idum-k*IQ)-IR*k
	if (idum.lt.0) idum=idum+IM
	if (j.le.NTAB) iv(j)=idum
   enddo
   iy=iv(1)
endif
k=idum/IQ
idum=IA*(idum-k*IQ)-IR*k
if (idum.lt.0) idum=idum+IM
j=1+iy/NDIV
iy=iv(j)
iv(j)=idum
ran1circ_2=min(AM*iy,RNMX)
return

end function ran1circ_2

end module