module ranphys

use definitions
use random
use exprandom
use phys
use io
use math
use geometry

contains

!================================================================!
!= randirec chooses a random direction for the neutron to go 	=!
!= after a interaction with the moderator. Seperate for 	=!
!=  isotropic and elastic scattering.				=!
!================================================================!

subroutine randirec(a1,a2)		! input two negative seeds
implicit none
real			:: theta,phi,RL
real,dimension(2)	:: rancirc_temp
real,dimension(3)	:: m,u,v,a,x
integer 		:: a1,a2

r_direc_old=r_direc

if (isotropic==1) then
theta=2.*3.14159*ran1theta(a1)
phi  =acos(1.-2.*ran1phi(a2))

r_direc(1)=cos(theta)*sin(phi)
r_direc(2)=sin(theta)*sin(phi)
r_direc(3)=cos(phi)
else

rancirc_temp=rancirc(seed8,seed9)

impact_angle=pi*0.5-asin(rancirc_temp(1))

scattering_angle=abs( pi-abs(atan( -sqrt(E/Ehydro)*1/tan(impact_angle) )) & 
			-atan(1/tan(impact_angle)) )

m(:)=r_direc_old(:)*cos(scattering_angle)

a(1)=301
a(2)=4910
a(3)=-138

if ( length(crossp(a,m))==0 ) then
 a(1)=103
 a(2)=-459
 a(3)=11
endif

u(:)=crossp(m,a)/length(crossp(m,a))

v(:)=crossp(m,u)/length(crossp(m,u))

RL=sin(scattering_angle)

x(:)=m+u(:)*RL*cos(rancirc_temp(2))+v(:)*RL*sin(rancirc_temp(2))

r_direc(:)=x(:)/length(x)

endif

end subroutine randirec

!================================================================!
!= randist chooses a random distance for the neutron to travel 	=!
!= before next interaction with the moderator			=!
!================================================================!

subroutine randist		! input seed (a1) and sigma (L)
implicit none


if (E.gt.Etermisk*1.01) then

	if (isotropic==1) then
	 E=(E-Etermisk)*0.5+Etermisk
	else
	 E=E*cos(impact_angle)**2+Ehydro*sin(impact_angle)**2
	endif

 call cross_section
if (cylinder==1) then
 sigma=7.947E19*cross
else
 sigma=7.515E19*cross
endif


 dist=expdev(seed3,sigma)

else

 E=Etermisk
 dist=expdev(seed3,sigma2)

endif

end subroutine randist

!================================================================!
!= Newneutron sets a starting point and a starting depth in the	=!
!=  moderator. If the first point is above the output surface, 	=!
!=  the simulate routine will handel it correctly.		=!
!= Other forms of starting conditions can be entered here.	=!
!= The simulation currently uses isotropic starting at the 	=!
!=  middel if the main moderator is simulated.			=!
!= The simulation uses uniformly distributed starting points on	=!
!=  the z=0 plane. All directions is in the z direction.	=!
!================================================================!

subroutine newneutron(a4,a5,a6)		! input seed (a4,a5) 
implicit none
integer :: a4,a5,a6
real			:: theta,phi,RL
real,dimension(2)	:: rancirc_temp
real,dimension(3)	:: m,u,v,a,x
integer 		:: a1,a2

r(:)=0
r_old(:)=0

if (cylinder==0) then

 if (absorption==0) then
  if (ran1E1(seed6).lt.0.615) then 	! 61.5 percent thermal
   E=Etermisk
  else
   E=E1*lindev(seed7)/sqrt(2.)		! rest is linear distributed.
  endif

  call cross_section
  sigma=7.515E19*cross

  r(1)=0.98*xmax*ran1x(a4)+0.01*xmax
  r_old(1)=r(1)
  r(2)=0.98*ymax*ran1y(a5)+0.01*ymax
  r_old(2)=r(2)
  r(3)=expdev(a6,sigma)+0.001
  dist=r(3)
  r_old(3)=0.01
  tdist=r(3)
  r_direc(1)=0
  r_direc(2)=0
  r_direc(3)=1
 else

   E=Etermisk
   tdistmax=expabs(seed10,sigmaabs)

  call cross_section
  sigma=7.515E19*cross

  r(1)=0.98*xmax*ran1x(a4)+0.01*xmax
  r_old(1)=r(1)
  r(2)=0.98*ymax*ran1y(a5)+0.01*ymax
  r_old(2)=r(2)
  r(3)=expdev(a6,sigma)+0.001
  dist=r(3)
  r_old(3)=0.01
  tdist=r(3)
  r_direc(1)=0
  r_direc(2)=0
  r_direc(3)=1
 endif

else

E=E1

 r(1)=xmax*0.5
 r_old(1)=r(1)+0.001
 r(2)=ymax*0.5
 r_old(2)=r(2)+0.001
 r(3)=zmax*0.5
 dist=0
 r_old(3)=r(3)+0.001
 tdist=0
 r_direc(1)=0
 r_direc(2)=0
 r_direc(3)=1
 
 theta=2.*3.14159*ran1theta(a1)
 phi  =acos(1.-2.*ran1phi(a2))

 r_direc(1)=cos(theta)*sin(phi)
 r_direc(2)=sin(theta)*sin(phi)
 r_direc(3)=cos(phi)

 call cross_section
 sigma=7.947E19*cross
 dist=expdev(seed3,sigma)
 tdist=tdist+dist
 interact=interact+1


 call newpos(0)

endif

end subroutine newneutron

end module ranphys